/************************************************************************
 * 			MEKATRONIX Copyright 1998 		       	*
 * Title        MuxSentk.c                                              *
 * Programmer   Keith L. Doty                                           *
 * Date         July 16, 1998                                           *
 * Version      1                                                       *
 *                                                                      *
 * Description								*
 *	This file contains functions to read the TALRIK sensors.        *
 ************************************************************************/


/**************************** Includes **********************************/

#include <hc11.h>
#include <mil.h>
#include <analog.h>

/************************************************************************/


/**************************** Constants *********************************/

#define NIRDT  14          /* Number of multiplexed IR detectors */
#define NCDS   6          /* Number of multiplexed CDS sensors */
#define NXA    6          /* Number of multiplexed eXtra analog sensors */

#define SEL_field 0x1f    /* Ones specify SEL field bit positions */

/*Multiplexer-Motor Direction Register MMR*/
#define MMR   *(unsigned char *)(0xffb8)

/* Define the MMR_SEL field hex values which multiplexes the sensor
   input into its assigned A-to-D channel. 
*/

/*Analog channel on PE0 from here to end of IRD?_SEL #defines */

#define	ECDSR_SEL	0x0a
#define	ECDSM_SEL	0x0b
#define	ECDSL_SEL	0x0c
#define	NCDSR_SEL	0x0d
#define	NCDSM_SEL	0x0e
#define	NCDSL_SEL	0x0f

#define	Charge_Volts_SEL	0x10  /*Not an MRSX01 connector*/
#define	Battery_Volts_SEL	0x11   /*Not an MRSX01 connector*/


#define	IRDR_SEL	0x12
#define	IRDBR_SEL	0x13
#define	IRDBMR_SEL	0x14
#define	IRDBML_SEL	0x15
#define	IRDBL_SEL	0x16
#define	IRDL_SEL	0x17
#define	IRDLW_SEL	0x08
#define	IRDRW_SEL	0x09

#define	IR3clk_SEL	0x12
#define	IR4clk_SEL	0x13
#define	IRD5clk_SEL	0x14
#define	IR7clk_SEL	0x15
#define	IR8clk_SEL	0x16
#define	IR9clk_SEL	0x17

/*Analog Channel PE1 from here down to Analog6*/

#define	R_Bump_SEL	0x10
#define	F_Bump_SEL	0x11

/*Xanalog refers to the ANALOG connector on the MRSXO1*/
#define	Analog1_SEL	0x12
#define	Analog2_SEL	0x13
#define	Analog3_SEL	0x14
#define	Analog4_SEL	0x15
#define	Analog5_SEL	0x16
#define	Analog6_SEL	0x17


/* TALRIK Users Manual sensor names as indicies of IRDT[]. The offset between array
   IRDT[i-1] and its corresponding physical connector name IRDTi is due to zero based
   indexing in the C language.
*/
			 /*IR Detector on the */
#define IRDL 	11	/*Left IRDT12*/
#define IRDFL 	1  	/*Front Left IRDT2*/
#define IRDFML 	2 	/*Front Middle Left IRDT3*/
#define IRDFM 	3 	/*Front Middle IRDT4*/
#define IRDFMR 	4 	/*Front Middle Right IRDT5*/
#define IRDFR 	0 	/*Front Right IRDT1*/
#define IRDR 	6 	/* Right IRDT7*/
#define IRDBL 	10 	/*Back Left IRDT11*/
#define IRDBML 	9		/*Back Middle Left IRDT10*/
#define IRDBM 	5		/*Back Middle IRDT6*/
#define IRDBMR 	8		/*Back Middle Right IRDT9*/
#define IRDBR 	7		/*Back Right IRDT8*/
#define IRDLW 	12	/*Back Left Wheel (Optional) IRDT13*/
#define IRDRW 	13	/*Back Right Wheel (Optional)IRDT14*/

/* TALRIK sensor clock position indicies into the IRDT array*/
			
#define IR1clk 	4   /* IR sensor at one o'clock is IRDT[IR1clk]=IRDT[4], etc*/
#define IR2clk 	0  	
#define IR3clk 	6 	  
#define IR4clk	7 	  
#define IR5clk 	8 	
#define IR6clk 	5 	
#define IR7clk 	9 	
#define IR8clk 	10 	
#define IR9clk 	11	
#define IR10clk 1		
#define IR11clk 2		
#define IR12clk 3  /* IR sensor at Twelve o'clock is IRDT[IR12clk]=IRDT[3] */


			 /*Nose CDS cells*/
#define NCDSL CDS[5] 		/*Left*/
#define NCDSM CDS[4]		/*Middle*/
#define NCDSR CDS[3]		/*Right*/

			 /*Eye CDS cells*/
#define ECDSL CDS[2] 		/*Left*/
#define ECDSM CDS[1] 		/*Middle*/
#define ECDSR CDS[0] 		/*Right*/

/* 
  Caution! Mekatronix reserves the following for future sensor assignments
  which may not be compatible to user sensor assignments.

*/

#define Analog1 Xanalog[0]
#define Analog2 Xanalog[1]
#define Analog3 Xanalog[2]
#define Analog4 Xanalog[3]
#define Analog5 Xanalog[4]
#define Analog6 Xanalog[5]



/************************************************************************/


/*************************** Prototypes *********************************/

void MMR_bit_clear(int);	/* Clear specified 1-bits in MMR.SEL */
void MMR_bit_set(int);		/* Set specified 1-bits in MMR.SEL */
void mux_sensor(int);		/* Load specified field into MMR.SEL */

void read_IR(void);		/*Reads all the TALRIK IR sensor inputs*/
void read_CDS(void);		/*Reads all the TALRIK CDS sensor inputs*/
void read_Xanalog(void);	/*Reads all the TALRIK sensors connected
					  the 6-pin Analog port of the MRSX01 board.	
					*/

unsigned int rear_bumper(void);	/*Returns rear bumper value*/
unsigned int front_bumper(void);	/*Returns front bumper value*/
unsigned int charge_volts(void);	/*Returns the charging voltage level*/
unsigned int battery_level(void);	/*Returns the battery voltage level*/

/************************************************************************/


/***************************** Globals **********************************/

unsigned int IRDT[NIRDT], CDS[NCDS], Xanalog[NXA];

/* Buffer memory of the MMR at(0xffb8). Mirrors contents of write-only MMR */
 char MMR_mirror; 

/************************************************************************/

void MMR_bit_clear(int bit_clear)
/**********************************************************************
 * Function description:                                              *
 *   Clear the bits in the MMR (Multiplexer/Motor Direction Register) *
 *   specified by the ones in bit_clear. Other bits unaffected. The   *
 *   MMR_mirror duplicates contents of the MMR which is write only.   *
 *   MMR_mirror allows the programmer to view the contents of the MMR.*
 * Returns:                                                           *
 *                                                                    *
 * Inputs                                                             *
 *   Parameters: bit_clear                                            *
 *   Globals:    MMR_mirror                            	 	      *
 *   Registers:  None                                                 *
 * Outputs                                                            *
 *   Parameters: None                                                 *
 *   Globals:    MMR_mirror, MMR                                      *
 *   Registers:  None                                                 *
 * Functions called: CLEAR_BIT				              *
 * Notes: None                                                        *
 **********************************************************************/
{
     CLEAR_BIT(MMR_mirror, bit_clear);
     MMR = MMR_mirror;
}
/***********************End MMR_bit_clear *******************************/


void MMR_bit_set(int bit_set)
/************************************************************************
 * Function description:                                              	*
 *   Set the bits in the MMR (Multiplexer/Motor Direction Register)   	*
 *   specified by the ones in bit_set. Other bits unaffected. The     	*
 *   MMR_mirror duplicates contents of the MMR which is write only.   	*
 *   MMR_mirror allows the programmer to view the contents of the MMR.	*
 * Returns:                                                           	*
 *                                                                    	*
 * Inputs                                                             	*
 *   Parameters: bit_clear                                            	*
 *   Globals:    MMR_mirror						*
 *   Registers:  None                                           	*
 * Outputs                                                            	*
 *   Parameters: None                                                 	*
 *   Globals:    MMR_mirror, MMR                                      	*
 *   Registers:  None                                                 	*
 * Functions called: SET_BIT			                  	*
 * Notes: None                                                        	*
 ************************************************************************/
{
     SET_BIT(MMR_mirror, bit_set);
     MMR = MMR_mirror;
 }
/***********************End MMR_bit_set ******************************/


void mux_sensor(int bit_field)
/************************************************************************
 * Function description:                                              	*
 *   Loads bit_field into the 5-bit SEL field of the MMR register	*
 *  (Multiplexer/Motor Direction Register), the first 3 bits unaffected.*
 *   The MMR_mirror duplicates contents of the MMR which is write only.	* 
 *   MMR_mirror allows the programmer to view the contents of the MMR.	*	
 * Returns:                                                           	*
 *                                                                    	*
 * Inputs                                                             	*
 *   Parameters: bit_field                                            	*
 *   Globals:    MMR_mirror						*
 *   Registers:  None                                                 	*
 * Outputs                                                            	*
 *   Parameters: None                                                 	*
 *   Globals:    MMR_mirror, MMR                                      	*
 *   Registers:  None                                                 	*
 * Functions called: CLEAR_BIT, SET_BIT		                  	*
 * Notes: None                                                        	*
 ************************************************************************/
{
/* Zero top 3 bits of bit_field so the corresponding bits in MMR
   are unaffected by this funciton. */
     bit_field  = bit_field & SEL_field;  

     CLEAR_BIT(MMR_mirror, SEL_field);    /* Clear the SEL field of the MMR */
     SET_BIT(MMR_mirror, bit_field); /* Set the appropriate bits in MMR.SEL */
     MMR = MMR_mirror;   	     /* Load MMR with MMR_mirror */
 }
/***********************End mux_sensor ******************************/


void read_IR(void)
/************************************************************************
 * Function: Reads all TALRIK IR sensors. 				*
 * Returns: None                                                        *
 *                                                                      *
 * Inputs                                                               *
 *   Parameters: None                                                   *
 *   Globals:    IRTD[NIRDT], MMR_mirror				*
 * Outputs                                                              *
 *   Parameters: 							*
 *   Globals:    IRTD[NIRDT], MMR_mirror, MMR				*
 *                                                                      *
 * Registers: None				                        *
 * Functions called: mux_sensor(), analog()                             *
 * Notes: This routine executes in about 1msec. It does not interfere   *
 *        with "auto" AD conversions in TOC5_isr.			*
 ************************************************************************/
{
int i;

/* Read IRDT1 through IRDT5 on the Front of TALRIK and IRDT6 on the back */
  for(i=0; i < 6; i++) IRDT[i] = analog(i+2);
 
/* Read IRDT7 to IRDT12 */

/* Initialize MMR_mirror to binary ???10010 */
  mux_sensor(IRDR_SEL);

  for(i=0; i < NIRDT-8; i++) 
     {
       IRDT[i+6] = analog(0);
       MMR_mirror += 1;
       MMR = MMR_mirror;
     }
  mux_sensor(IRDLW_SEL); IRDT[12] = analog(0);
  mux_sensor(IRDRW_SEL); IRDT[13] = analog(0);
}
/***********************End of read_IR **********************************/

void read_CDS(void)
/************************************************************************
 * Function: Reads all six CDS cells.				       	*
 * Returns: None                                                        *
 *                                                                      *
 * Inputs                                                               *
 *   Parameters: None                                                   *
 *   Globals:    CDS[NCDS], MMR_mirror			               	*
 * Outputs                                                              *
 *   Parameters: None                                                   *
 *   Globals:    CDS[NCDS], MMR_mirror, MMR				*
 *                                                                      *
 * Registers: None			                                *
 * Functions called: mux_sensor(), analog()           			*
 * Notes: 								*
 ************************************************************************/
{
int i;

/* Select the first CDS cell on the multiplexed PE0 channel. */
  mux_sensor(ECDSR_SEL);

  for(i=0; i < NCDS; i++) 
     {
       CDS[i] = analog(0);
       MMR_mirror += 1;
       MMR = MMR_mirror;
     }
}
/***********************End of read_CDS ****************************/


void read_Xanalog(void)
/************************************************************************
 * Function: Reads all six extra Analog cells.			       	*
 * Returns: None                                                        *
 *                                                                      *
 * Inputs                                                               *
 *   Parameters: None                                                   *
 *   Globals:    Xanalog[NXA], MMR_mirror			       	*
 * Outputs                                                              *
 *   Parameters: None                                                   *
 *   Globals:    Xanalog[NXA], MMR_mirror, MMR				*
 *                                                                      *
 * Registers: None			                                *
 * Functions called: mux_sensor(), analog()           			*
 * Notes: 								*
 ************************************************************************/
{
int i;
/* Select the first extra analog cell on the multiplexed PE1 channel. */
  mux_sensor(Analog1_SEL);

  for(i=0; i < NXA; i++) 
     {
       Xanalog[i] = analog(1);
       MMR_mirror += 1;
       MMR = MMR_mirror;
     };
}
/***********************End of read_Xanalog ****************************/



unsigned int rear_bumper(void)
/************************************************************************
 * Function:  Reads the rear bump sensor output.		 	*
 * Returns: Rear bumper value                                           *
 *                                                                      *
 * Inputs                                                               *
 *   Parameters: None                                                   *
 *   Globals:    MMR_mirror						*
 * Outputs                                                              *
 *   Parameters: None                                                   *
 *   Globals:    MMR_mirror, MMR					*	
 *                                                                      *
 * Registers: None							*
 * Functions called: mux_sensor() analog()			        *
 * Notes:  								*
 ************************************************************************/
{
/* Select the rear bumper on the multiplexed PE1 AD channel */
  mux_sensor(R_Bump_SEL);
  return analog(1);
}
/***********************End of rear_bumper******************************/

unsigned int front_bumper(void)
/************************************************************************
 * Function:  Reads the front bumper.					*
 * Returns: Front bumper value                                          *
 *                                                                      *
 * Inputs                                                               *
 *   Parameters: None                                                   *
 *   Globals:    MMR_mirror						*
 * Outputs                                                              *
 *   Parameters: None                                                   *
 *   Globals:    MMR_mirror, MMR		        		*
 *                                                                      *
 * Registers: None			                                *
 * Functions called: mux_sensor, analog()  				*
 * Notes:								*
 ************************************************************************/
{
/* Select the front bumper on the multiplexed PE1 AD channel */
  mux_sensor(F_Bump_SEL);

/* Read the Battery charging voltage */
  return analog(1);
}
/***********************End of front_bumper ****************************/


unsigned int charge_volts(void)
/************************************************************************
 * Function:  Reads the battery charging volts.				*
 * Returns: The charger voltage			                        *
 *                                                                      *
 * Inputs                                                               *
 *   Parameters: None                                                   *
 *   Globals:    MMR_Mirror  				                *
 * Outputs                                                              *
 *   Parameters: None                                                   *
 *   Globals:    MMR_mirror, MMR					*
 *                                                                      *
 * Registers: None			                                *
 * Functions called: mux_sensor, analog()           			*
 * Notes:								*
 ************************************************************************/
{
/* Select the voltage charger line on the multiplexed PE0 AD channel */
  mux_sensor(Charge_Volts_SEL);

/* Read the battery charging voltage */
  return analog(0);
}
/***********************End of charge_volts ****************************/


unsigned int battery_level(void)
/************************************************************************
 * Function:  Reads the battery voltage level.				*
 * Returns: Battery voltage level                                       *
 *                                                                      *
 * Inputs                                                               *
 *   Parameters: None                                                   *
 *   Globals:    MMR_mirror  				                *
 * Outputs                                                              *
 *   Parameters: None                                                   *
 *   Globals:    MMR_mirror, MMR					*
 *                                                                      *
 * Registers: None			                                *
 * Functions called: mux_sensor, analog()           			*
 * Notes:								*
 ************************************************************************/
{
/* Select the battery voltage on the multiplexed PE0 AD channel */
  mux_sensor(Battery_Volts_SEL);

/* Read the battery charging voltage */
  return analog(0);
}
/***********************End of battery_level ****************************/


