/**********************************************************************
 * Title        servotk.c  (Servo Drivers for Talrik)                 *
 * Programmer   Drew Bagnell, Talrik drivers                          *
 * Date         July, 1998                                            *
 * Version      1                                                     *
 *                                                                    *
 * Description: Servo drivers based on the TJ/TJP standard drivers    *
 *                                                                    *
 **********************************************************************/


/**************************** Includes **********************************/
#include <hc11.h>
#include <mil.h>
/************************************************************************/


/**************************** Constants *********************************/
#define HALFPERIOD 20000         /* 50Hz refresh for each servo */
/************************************************************************/


/*************************** Prototypes *********************************/
#pragma interrupt_handler servo_hand
void init_servos(void);
void servo(int, int);
void servo_hand();
/************************************************************************/


/***************************** Globals **********************************/
unsigned int width[2];       /* width[] is the on-time for each servo*/
unsigned int current_width;
char signal_state, servomask[2];
/************************************************************************/



void 
init_servos(void)
/************************************************************************
 * Function: Initializes all necessary variables for the servos.        *
 * Returns: None                                                        *
 *                                                                      *
 * Inputs                                                               *
 *   Parameters: None                                                   *
 *   Globals:    None                                                   *
 *   Registers:  None                                                   *
 * Outputs                                                              *
 *   Parameters: None                                                   *
 *   Globals:   width[], signal_state, servomask[], current_width       *
 *   Registers: TOC1, PORTB, TMSK1, TCTL1                               *
 * Functions called: None                                               *
 * Notes: None                                                          *
 ************************************************************************/
{

  INTR_OFF();
  
  //install interrupt handler on OC1
  *((void (**)())0xffe8)  = servo_hand;
  
  CLEAR_BIT(OC1M,0xff);          /* Interrupt will not affect OC pins */

  width[0] = width[1] =  0;  /* Motors start turned off */
  TOC1 = 0;                       
  signal_state = 0;
  current_width = 0;
  servomask[0] = 0x10;                  // modified servo masks oc4
  servomask[1] = 0x80;                  // oc1

  /* PA gates on  */
  SET_BIT(PACTL,0x80);                  /* SET PORTA, BIT7 (OC1) to output */
  SET_BIT(TMSK1,0x80);                  /* Enable OC1 interrupt */
  INTR_ON();
}


void
servo(int index, int newwidth)
/************************************************************************
 * Function: Sets one of the servos to the speed defined by newwidth    *
 * Returns: None                                                        *
 *                                                                      *
 * Inputs                                                               *
 *   Parameters: index, newwidth                                        *
 *   Globals:    None                                                   *
 *   Registers:  None                                                   *
 * Outputs                                                              *
 *   Parameters: None                                                   *
 *   Globals: width[index]                                              *
 *   Registers:  None                                                   *
 * Functions called: None                                               *
 * Notes: None                                                          *
 ************************************************************************/
{
  asm("ldd %index \n"
      "aslb \n"
      "ldy #_width \n"
      "aby \n"
      "ldd %newwidth \n"
      "std 0,y");
}



void 
servo_hand()
/************************************************************************
 * Function: Interrupt handler for servo signals.                       *
 *	     Interrupt is OC1. Servo_1 is on PA4 and Servo_2 on PA7.    *
 * Returns: None                                                        *
 *                                                                      *
 * Inputs                                                               *
 *   Parameters: None                                                   *
 *   Globals:    width[], signal_state, servomask[], current_width      *
 *   Registers:  None                                                   *
 * Outputs                                                              *
 *   Parameters: None                                                   *
 *   Globals:    current_width                                          *
 *   Registers:  TOC1, PORTA, TFLG1                                     *
 * Functions called: None                                               *
 * Notes: None                                                          *
 ************************************************************************/
{
  char odd, index;
  unsigned int pwidth;

/*
   signal_state = 0  -> Turn on servo0
   signal_state = 1  -> Turn off servo0
   signal_state = 2  -> Turn on servo1
   signal_state = 3  -> Turn off servo1
*/

  signal_state &= 0x03;         /* Only use last 2 bits */
  index = (signal_state >> 1);  /* index references current servo */

  odd = (signal_state & 0x01);

  pwidth = *(width+index);
  
  if ((pwidth == 0)&&(!(odd)))
  {
    TOC1 += HALFPERIOD;
    signal_state++;
  }
  else
  {
    if (odd)
      TOC1 += (HALFPERIOD - current_width);
    else
      TOC1 += pwidth;
  
    PORTA ^= *(servomask+index);
  }

  current_width = pwidth;
  signal_state++;
  if (signal_state == 4)
    signal_state = 0;
  
   CLEAR_FLAG(TFLG1,0x80);          /* Clear OC1I flag */
}
